!---------------------------------------------------------------------------------
!  This program converts a subdata_*.csv file into monthly values.
!  Written by Tim Hunter in late 2019.
! 
!  input file(s):  subdata_xxxnn.csv or
!                  subdata_xxx_aaaa.csv
! 
!  output files:   xxxnn_tmn_mon.csv   Minimum Air Temperature
!                  xxxnn_tmx_mon.csv   Maximum Air Temperature
!                  xxxnn_prc_mon.csv   Precipitation
!                  xxxnn_tmp_mon.csv   Mean Overlake Air Temperature
!                  xxxnn_dew_mon.csv   Mean Overlake Dew Point Temperature
!                  xxxnn_wnd_mon.csv   Mean Overlake Wind Speed
!                  xxxnn_cld_mon.csv   Mean Overlake Cloud Cover
!                     -- or --
!                  xxx_aaaa_tmn_mon.csv   Minimum Air Temperature
!                  xxx_aaaa_tmx_mon.csv   Maximum Air Temperature
!                  xxx_aaaa_prc_mon.csv   Precipitation
!                  xxx_aaaa_tmp_mon.csv   Mean Overlake Air Temperature
!                  xxx_aaaa_dew_mon.csv   Mean Overlake Dew Point Temperature
!                  xxx_aaaa_wnd_mon.csv   Mean Overlake Wind Speed
!                  xxx_aaaa_cld_mon.csv   Mean Overlake Cloud Cover
!
!  where xxx  = 3-character lake code (sup, mic, hur, etc)
!        nn   = 2-digit subbasin number (00, 01, 02, ...)
!        aaaa = area code (land, basn, lake)
!
!  Note that I am utilizing the code from GLSHFS for reading the GLSHFS files.
!  That requires a little extra overhead, but saves me from "reinventing the wheel".
!  I am also grabbing various other bits and pieces from the GLSHFS code, sometimes
!  with small modifications for this program.  It keeps things relatively 
!  stand-alone, but without having to do *everything* from scratch.
!
!---------------------------------------------------------------------------------

!==============================================================================
!==============================================================================
!==============================================================================
      PROGRAM glshfs_submet_to_tblm
      USE GL_Constants
      USE MyKinds                     ! part of glshfs_util.f90
      USE GLSHFS_Util
      USE GLSHFS_Files
      USE DailyDataStructures
      USE GlerlDataTypesAndUnits
      IMPLICIT NONE

      INTEGER :: IOS, I, K, ArgC
      INTEGER :: Sub, Sub1, Sub2, LkNum
      INTEGER :: D, M, Y, SD, SM, SY, ED, EM, EY
      INTEGER :: Seq, SSeq, ESeq
      INTEGER :: UserStartYear, UserEndYear
      INTEGER, DIMENSION(7) :: UNum
      INTEGER, DIMENSION(:,:,:), ALLOCATABLE :: AccCnt
      LOGICAL :: OK
      REAL, DIMENSION(7) :: Raw
      REAL, DIMENSION(:,:,:), ALLOCATABLE :: AccTot, MonOut

      CHARACTER (LEN=3)   :: Bsn
      CHARACTER (LEN=4)   :: S4
      CHARACTER (LEN=10)  :: SB
      CHARACTER (LEN=80)  :: FNameTMin, FNameTMax, FNamePrec, FNameTAvg, FNameDewp
      CHARACTER (LEN=80)  :: FNameWind, FNameClwd, FNameMet, S
      CHARACTER (LEN=80)  :: CLine
      CHARACTER (LEN=10), DIMENSION(12) :: OutStr
      CHARACTER (LEN=45), DIMENSION(5)  :: ArgS

      !
      !  1 = Monthly value is TOTAL of daily values
      !  2 = Monthly value is MEAN of daily values
      !  order is:
      !    TMin, TMax, Prec, TAvg, Dewp, Wind, Cloud
      ! 
      INTEGER, DIMENSION(7), PARAMETER :: DT_Method = (/2, 2, 1, 2, 2, 2, 2/)
       
      TYPE (TDlyData), POINTER   :: TDDP1, TDDP2, TDDP3, TDDP4, TDDP5, TDDP6, TDDP7
      TYPE (TDlyDataForSubbasin) :: SubData

      !
      ErrorLevel = 0
      ErrorMessage = ''
      CALL ClearFilesOpen()
      SubData = TDlyDataForSubbasin()

      !
      !  Get input from user [lake subbasin]
      !  GET_COMMAND() counts program name as the first argument
      !
      CALL GET_COMMAND(CLine)
      CALL ParseCmdLine(CLine, Args, ArgC)      
      IF (ArgC .LT. 3) GOTO 875
      
      !
      Bsn = TRIM(Args(2))
      LkNum = LakeNumberFromName3(Bsn)
      IF (LkNum .LT. 1) THEN
         PRINT*, 'Error with lake name. ('//TRIM(Bsn)//')'
         GOTO 875
      END IF
      CALL Lowercase(Bsn)
      
      !      
      SB = TRIM(Args(3))
      CALL Lowercase(SB)
      IF (TRIM(SB) .EQ. 'all') THEN
         Sub1 = 0
         Sub2 = NumSubbasins(LkNum)
      ELSE IF (TRIM(SB) .EQ. 'lake') THEN
         Sub1 = 0
         Sub2 = 0
      ELSE IF (TRIM(SB) .EQ. 'land') THEN
         Sub1 = 98
         Sub2 = 98
      ELSE IF (TRIM(SB) .EQ. 'basn') THEN
         Sub1 = 99
         Sub2 = 99
      ELSE
         READ(SB, *, IOSTAT=IOS) Sub1
         IF (IOS .NE. 0) THEN
            PRINT*, 'Error with subbasin specifier (', TRIM(Args(3)), ')'
            PRINT*, 'Must be either "all", "land", "basn", or a subbasin number.'
            GOTO 875
         END IF
         Sub2 = Sub1
      END IF

      !
      UserStartYear = -9999
      UserEndYear   = -9999
      IF (ArgC .GE. 5) THEN
         S = TRIM(Args(4))
         READ(S, *, IOSTAT=IOS) Y
         IF (IOS .EQ. 0) THEN
            IF ((Y .GT. 1500) .AND. (Y .LT. 2500)) UserStartYear = Y
         END IF
         S = TRIM(Args(5))
         READ(S, *, IOSTAT=IOS) Y
         IF (IOS .EQ. 0) THEN
            IF ((Y .GT. 1500) .AND. (Y .LT. 2500)) UserEndYear = Y
         END IF
      END IF
      
      !
      !  Main loop     
      !
      DO Sub = Sub1, Sub2
         IF (Sub .LT. 90) THEN
            WRITE(FNameMet, 4001) Bsn, Sub
         ELSE IF (Sub .EQ. 98) THEN
            WRITE(FNameMet, 4002) Bsn, 'land'
         ELSE IF (Sub .EQ. 99) THEN
            WRITE(FNameMet, 4002) Bsn, 'basn'
         END IF
         INQUIRE(FILE=TRIM(FNameMet), EXIST=OK)
         IF (.NOT. OK) THEN
            PRINT*, 'Missing file (', TRIM(FNameMet), ')'
            PRINT*, 'Unable to perform conversion'
            GOTO 898
         END IF
         
         CALL ReadFile_OneSubbasin(TRIM(FNameMet), SubData);  IF (ErrorLevel .NE. 0) GOTO 899
         SSeq = SubData%GetStartDate()
         ESeq = SubData%GetEndDate()
         IF (UserStartYear .GT. 0) THEN
            CALL DateSequence(1, 1, UserStartYear, Seq);  IF (ErrorLevel .NE. 0) GOTO 899
            SSeq = MAX(SSeq, Seq)
         END IF
         IF (UserEndYear .GT. 0) THEN
            CALL DateSequence(31, 12, UserEndYear, Seq);  IF (ErrorLevel .NE. 0) GOTO 899
            ESeq = MIN(ESeq, Seq)
         END IF
         IF (ESeq .LT. SSeq) THEN
            PRINT*, 'Error: End date is earlier than start date.'
            GOTO 898
         END IF

         !
         !  Get pointers to data objects
         !
         TDDP1 => SubData%GetPointerToDataOfType(GDT_AirtempMin)
         TDDP2 => SubData%GetPointerToDataOfType(GDT_AirtempMax)
         TDDP3 => SubData%GetPointerToDataOfType(GDT_Precipitation)
         TDDP4 => SubData%GetPointerToDataOfType(GDT_AirtempMean)
         TDDP5 => SubData%GetPointerToDataOfType(GDT_DewpointMean)
         TDDP6 => SubData%GetPointerToDataOfType(GDT_WindSpeed)
         TDDP7 => SubData%GetPointerToDataOfType(GDT_CloudCover)

         !
         !  Open the 7 standard output files (tmin, tmax, precip, airtemp, dewpt, wind, cloud)
         !
         WRITE(S4, '(I2.2)') Sub
         IF (Sub .EQ. 98) S4 = 'land'
         IF (Sub .EQ. 99) S4 = 'basn'
         IF ((Sub .EQ. 0) .AND. (SB .EQ. 'lake')) S4 = 'lake'
         
         WRITE(FNameTmin, 4011) 'tmn', Bsn, TRIM(ADJUSTL(S4))
         WRITE(FNameTmax, 4011) 'tmx', Bsn, TRIM(ADJUSTL(S4))
         WRITE(FNamePrec, 4011) 'prc', Bsn, TRIM(ADJUSTL(S4))
         WRITE(FNameTavg, 4011) 'air', Bsn, TRIM(ADJUSTL(S4))
         WRITE(FNameDewp, 4011) 'dew', Bsn, TRIM(ADJUSTL(S4))
         WRITE(FNameWind, 4011) 'wnd', Bsn, TRIM(ADJUSTL(S4))
         WRITE(FNameClwd, 4011) 'cld', Bsn, TRIM(ADJUSTL(S4))

         UNum(1) = GetFreeUnitNumber()
         OPEN(UNIT=UNum(1), FILE=TRIM(FNameTmin), STATUS='REPLACE', ERR=811)
         CALL FileWasOpened(Unum(1))
         UNum(2)= GetFreeUnitNumber()
         OPEN(UNIT=UNum(2), FILE=TRIM(FNameTmax), STATUS='REPLACE', ERR=821)
         CALL FileWasOpened(UNum(2))
         UNum(3) = GetFreeUnitNumber()
         OPEN(UNIT=UNum(3), FILE=TRIM(FNamePrec), STATUS='REPLACE', ERR=831)
         CALL FileWasOpened(UNum(3))
         UNum(4) = GetFreeUnitNumber()
         OPEN(UNIT=UNum(4), FILE=TRIM(FNameTavg), STATUS='REPLACE', ERR=841)
         CALL FileWasOpened(UNum(4))
         UNum(5) = GetFreeUnitNumber()
         OPEN(UNIT=UNum(5), FILE=TRIM(FNameDewp), STATUS='REPLACE', ERR=851)
         CALL FileWasOpened(UNum(5))
         UNum(6) = GetFreeUnitNumber()
         OPEN(UNIT=UNum(6), FILE=TRIM(FNameWind), STATUS='REPLACE', ERR=861)
         CALL FileWasOpened(UNum(6))
         UNum(7) = GetFreeUnitNumber()
         OPEN(UNIT=UNum(7), FILE=TRIM(FNameClwd), STATUS='REPLACE', ERR=871)
         CALL FileWasOpened(UNum(7))

         !
         !  Write header lines
         !
         DO K = 1, 7
            WRITE(UNum(K), 5010) Bsn
            IF (Sub .EQ. 0) THEN
               WRITE(UNum(K), 5012) 
            ELSE IF (Sub .EQ. 98) THEN
               WRITE(UNum(K), 5013) 
            ELSE IF (Sub .EQ. 99) THEN
               WRITE(UNum(K), 5014) 
            ELSE
               WRITE(UNum(K), 5011) Sub
            END IF
         END DO
         WRITE(UNum(1), 5017) 'Tmin', 'Mean of the daily minimum air temperatures'
         WRITE(UNum(1), 5018) 'deg C'
         WRITE(UNum(2), 5017) 'Tmax', 'Mean of the daily maximum air temperatures'
         WRITE(UNum(2), 5018) 'deg C'
         WRITE(UNum(3), 5017) 'Precipitation', 'Sum of daily precipitation values'
         WRITE(UNum(3), 5018) 'mm'
         WRITE(UNum(4), 5017) 'Tavg', 'Mean of the daily average air temperatures'
         WRITE(UNum(4), 5018) 'deg C'
         WRITE(UNum(5), 5017) 'Dewpoint', 'Mean of the daily dewpoint temperatures'
         WRITE(UNum(5), 5018) 'deg C'
         WRITE(UNum(6), 5017) 'Windspeed', 'Mean of the daily average wind speeds - scalar'
         WRITE(UNum(6), 5018) 'deg C'
         WRITE(UNum(7), 5017) 'Cloud', 'Mean of the daily average cloud cover values'
         WRITE(UNum(7), 5018) 'Percent'
         DO K = 1, 7
            WRITE(UNum(K), 5019)
         END DO
         
         !
         !  Allocate the arrays
         !
         CALL SequenceDate(SD, SM, SY, SSeq)
         CALL SequenceDate(ED, EM, EY, ESeq)
         ALLOCATE(AccTot(SY:EY,12,7), AccCnt(SY:EY,12,7), MonOut(SY:EY,12,7), STAT=IOS)
         IF (IOS .NE. 0) THEN
            PRINT*, 'Error allocating arrays'
            CALL EXIT(1)
         END IF
         AccTot(:,:,:) = 0.0
         AccCnt(:,:,:) = 0
         MonOut(:,:,:) = MissingData_Real
         
         !
         !  Calculate the monthly values and write them into the output files
         !
         DO Seq = SSeq, ESeq
            CALL SequenceDate(D, M, Y, Seq)
            I = Seq - SSeq + 1
            Raw(1) = TDDP1%GetDataVal(I)
            Raw(2) = TDDP2%GetDataVal(I)
            Raw(3) = TDDP3%GetDataVal(I)
            Raw(4) = TDDP4%GetDataVal(I)
            Raw(5) = TDDP5%GetDataVal(I)
            Raw(6) = TDDP6%GetDataVal(I)
            Raw(7) = TDDP7%GetDataVal(I)
            DO K = 1, 7
               IF (Raw(K) .GT. MissingData_Real_Test) THEN
                  AccTot(Y, M, K) = AccTot(Y, M, K) + Raw(K)
                  AccCnt(Y, M, K) = AccCnt(Y, M, K) + 1
               END IF
            END DO
         END DO
          
         DO Y = SY, EY
            DO M = 1, 12
               DO K = 1, 7
                  IF (AccCnt(Y,M,K) .GT. 10) THEN
                     IF (DT_Method(K) .EQ. 1) THEN
                        MonOut(Y,M,K) = AccTot(Y,M,K)                  ! total (e.g. precip)
                     ELSE
                        MonOut(Y,M,K) = AccTot(Y,M,K) / AccCnt(Y,M,K)  ! mean (e.g. Tmax)
                     END IF
                  END IF
               END DO
            END DO
            DO K = 1, 7
               DO M = 1, 12 
                  IF (MonOut(Y,M,K) .GT. MissingData_Real_Test) THEN
                     WRITE(OutStr(M), 5001) MonOut(Y,M,K)
                  ELSE
                     OutStr(M) = '        NA'
                  END IF
               END DO
               WRITE(UNum(K), 5002) Y, (OutStr(M), M=1,12)
            END DO
         END DO

         !
         !  Close the output files
         !
         DO K = 1, 7
            CLOSE(UNum(K))
            CALL FileWasClosed(UNum(K))
         END DO

         !
         !  Clean up for next subbasin
         !
         DEALLOCATE(MonOut, AccTot, AccCnt, STAT=IOS)
      END DO         
      
      GOTO 999

!
!  Handlers for I/O errors
!
 811  ErrorMessage = 'Error opening file ' // TRIM(FNameTmin)
      CALL PassMsg; GOTO 898
 821  ErrorMessage = 'Error opening file ' // TRIM(FNameTmax)
      CALL PassMsg; GOTO 898
 831  ErrorMessage = 'Error opening file ' // TRIM(FNamePrec)
      CALL PassMsg; GOTO 898
 841  ErrorMessage = 'Error opening file ' // TRIM(FNameTavg)
      CALL PassMsg; GOTO 898
 851  ErrorMessage = 'Error opening file ' // TRIM(FNameDewp)
      CALL PassMsg; GOTO 898
 861  ErrorMessage = 'Error opening file ' // TRIM(FNameWind)
      CALL PassMsg; GOTO 898
 871  ErrorMessage = 'Error opening file ' // TRIM(FNameClwd)
      CALL PassMsg; GOTO 898


 875  PRINT*, 'USAGE: glshfs_submet_to_tblm  bsn  sub  [StartYear EndYear]'
      PRINT*, '       bsn = sup, mic, hur, etc'
      PRINT*, '       sub = 0..n  or  "all" or "lake" or "land" or "basn"'
      PRINT*, '  e.g. glshfs_submet_to_tblm sup 00'  
      PRINT*, '       glshfs_submet_to_tblm sup all'
      PRINT*, '       glshfs_submet_to_tblm sup land'
      PRINT*, '       glshfs_submet_to_tblm sup 10  1950 1990'
      CALL EXIT(1)

      
 898  ErrorLevel = 1
 899  ErrorMessage = '[traceback] : glshfs_submet_to_tblm [main]...'
      CALL PassMsg
      GOTO 999


!
!  Final cleanup
!
      DEALLOCATE(MonOut, AccTot, AccCnt, STAT=IOS)
  999 CALL CloseAllFiles()

!
!  FORMAT statements
!

 4001 FORMAT('subdata_', A3,    I2.2,  '.csv')
 4002 FORMAT('subdata_', A3, '_', A4,  '.csv')
 4011 FORMAT(A3, '_', A3, '_', A, '_mon.csv')
 
 5001 FORMAT(F10.3)
 5002 FORMAT(I4.4, 12(',', A))
 
 5010 FORMAT('Monthly data for lake:, ', A3)
 5011 FORMAT('Subbasin:,', I2.2)
 5012 FORMAT('Subbasin:, Overlake')
 5013 FORMAT('Subbasin:, Overland')
 5014 FORMAT('Subbasin:, Overbasin (lake+land)')

 5017 FORMAT('DataType:, ', A)
 5018 FORMAT('Units:, ', A)
 5019 FORMAT('YYYY,       Jan,       Feb,       Mar,       Apr,       May,       Jun,',    &
                  '       Jul,       Aug,       Sep,       Oct,       Nov,       Dec')
 
      END PROGRAM


