!---------------------------------------------------------------------------------
!  This program converts a summary_*.csv file into monthly values.
!  Written by Tim Hunter in late 2019.
! 
!  input file(s):  Specified by user, but typically:
!                  summary_xxx.csv or
!                  summary_xxxnn.csv
! 
!  output files:   Also specified by user, recommended names:
!                  summary_xxx_mon.csv or
!                  summary_xxxnn_mon.csv
!
!  where xxx  = 3-character lake code (sup, mic, hur, etc)
!        nn   = 2-digit subbasin number (00, 01, 02, ...)
!
!  Note that I am utilizing the code from GLSHFS for reading the GLSHFS files, with
!  some minor modifications and simplifications.
!  That requires a little extra overhead, but saves me from "reinventing the wheel".
!  I am also grabbing various other bits and pieces from the GLSHFS code, sometimes
!  with small modifications for this program.  It keeps things relatively 
!  stand-alone, but without having to do *everything* from scratch.
!
!---------------------------------------------------------------------------------
MODULE glsummary2mon_subs
      USE GLSHFS_Util
      USE GlerlDataTypesAndUnits

      CHARACTER (LEN=3), PARAMETER :: LakeName3(7) =           &
         (/'sup', 'mic', 'hur', 'geo', 'stc', 'eri', 'ont'/)
      CHARACTER(LEN=10), PARAMETER :: LakeName10(7) =          &
            (/'Superior  ', 'Michigan  ', 'Huron     ',        &
              'Georgian  ', 'StClair   ', 'Erie      ',        &
              'Ontario   '/)
      INTEGER, PARAMETER :: NumSubbasins(7) =                  &
         (/22, 27, 16, 13, 7, 21, 15/)
         
      !
      !  This structured type is a generic container for header information.
      !  It is set up to hold header info from ANY of the predefined formats.
      !
      !  The variable named FormatType tells which fields hold actual
      !  defined data values. For example, if the FormatType variable is
      !  assigned the value HFT_SingleStation, then the fields StnID, StnName,
      !  Latitude, Longitude, DataTypes, DataUnits will all be defined. The
      !  fields for basin, subbasin number, etc will be undefined.
      !
      !  The subroutine that fills this structure will need to be informed
      !  which format type to expect.
      !
      !  The format types are also defined here.
      !
      INTEGER, PARAMETER, PUBLIC :: HFT_Undefined        =   -1
      INTEGER, PARAMETER, PUBLIC :: HFT_SingleStation    = 7001
      INTEGER, PARAMETER, PUBLIC :: HFT_SingleSubbasin   = 7002
      INTEGER, PARAMETER, PUBLIC :: HFT_SingleLake       = 7003
      INTEGER, PARAMETER, PUBLIC :: HFT_MultiStation     = 7004
      INTEGER, PARAMETER, PUBLIC :: HFT_MultiSubbasin    = 7005
      INTEGER, PARAMETER, PUBLIC :: HFT_LBRM             = 7006
      TYPE, PUBLIC :: THeaderInfoType
         LOGICAL :: InfoIsValid
         INTEGER :: FormatType, NumLines
         INTEGER :: SDate, EDate, NumDays, NewDataDate
         INTEGER :: SubbasinNumber
         INTEGER :: NumDataColumns
         REAL    :: Elevation, Area
         CHARACTER(LEN=3) :: Bsn
         INTEGER,               DIMENSION(:), ALLOCATABLE :: DataType, DataUnit
         REAL,                  DIMENSION(:), ALLOCATABLE :: Latitude, Longitude, SubArea
         CHARACTER(LEN=9),      DIMENSION(:), ALLOCATABLE :: SubbasinID
         CHARACTER(LEN=25),     DIMENSION(:), ALLOCATABLE :: StnID
         CHARACTER(LEN=75),     DIMENSION(:), ALLOCATABLE :: StnName
      CONTAINS
         FINAL  :: DeleteHeaderInfo
         PROCEDURE, PASS(this) :: Clear   => ClearHeaderInfo
         PROCEDURE, PASS(this) :: PrintIt => PrintHeaderInfo
      END TYPE THeaderInfoType
      INTERFACE THeaderInfoType
         PROCEDURE CreateHeaderInfo
      END INTERFACE THeaderInfoType
         
      !
      !  We sometimes want a numeric "subbasin" number to use when designating
      !  the overland, overlake, or overbasin areas. Overlake has always been
      !  defined as subbasin 0, by convention. But there is no established
      !  convention for overland or overbasin. I have sometimes used 98 and 99
      !  for that, and it works ok. So I am defining those 2 here.
      !
      INTEGER, PARAMETER :: SubNum_OverLand  = 98
      INTEGER, PARAMETER :: SubNum_OverBasin = 99
         
CONTAINS

      !--------------------------------------------------------
      FUNCTION CreateHeaderInfo() RESULT(this)
         IMPLICIT NONE
         TYPE (THeaderInfoType) :: this
         CALL this%Clear()
      END FUNCTION CreateHeaderInfo
      
      !--------------------------------------------------------
      SUBROUTINE DeleteHeaderInfo(this)
         IMPLICIT NONE
         TYPE (THeaderInfoType) :: this
         CALL ClearHeaderInfo(this)
      END SUBROUTINE DeleteHeaderInfo

      !--------------------------------------------------------
      SUBROUTINE ClearHeaderInfo(this)
         IMPLICIT NONE
         CLASS (THeaderInfoType) :: this
         INTEGER :: IOS
         this%InfoIsValid    = .FALSE.
         this%FormatType     = HFT_Undefined
         this%NumLines       = 0
         this%SubbasinNumber = -1
         this%Elevation      = MissingData_Real
         this%Area           = MissingData_Real
         this%Bsn            = ''
         this%SDate          = MissingData_Date
         this%EDate          = MissingData_Date
         this%NumDays        = 0
         this%NumDataColumns = 0
         IF (ALLOCATED(this%DataType))   DEALLOCATE(this%DataType,   STAT=IOS)
         IF (ALLOCATED(this%DataUnit))   DEALLOCATE(this%DataUnit,   STAT=IOS)
         IF (ALLOCATED(this%Latitude))   DEALLOCATE(this%Latitude,   STAT=IOS)
         IF (ALLOCATED(this%Longitude))  DEALLOCATE(this%Longitude,  STAT=IOS)
         IF (ALLOCATED(this%SubArea))    DEALLOCATE(this%SubArea,    STAT=IOS)
         IF (ALLOCATED(this%SubbasinID)) DEALLOCATE(this%SubbasinID, STAT=IOS)
         IF (ALLOCATED(this%StnID))      DEALLOCATE(this%StnID,      STAT=IOS)
         IF (ALLOCATED(this%StnName))    DEALLOCATE(this%StnName,    STAT=IOS)
      END SUBROUTINE ClearHeaderInfo

      !--------------------------------------------------------
      SUBROUTINE PrintHeaderInfo(this)
         IMPLICIT NONE
         CLASS (THeaderInfoType) :: this
         WRITE(ErrorMessage, 1001)  this%InfoIsValid;     CALL PassMsg
         WRITE(ErrorMessage, 1002)  this%FormatType;      CALL PassMsg
         WRITE(ErrorMessage, 1003)  this%NumLines;        CALL PassMsg
         WRITE(ErrorMessage, 1004)  this%SubbasinNumber;  CALL PassMsg
         WRITE(ErrorMessage, 1005)  this%Elevation;       CALL PassMsg
         WRITE(ErrorMessage, 1006)  this%Area;            CALL PassMsg
         WRITE(ErrorMessage, 1007)  this%Bsn;             CALL PassMsg
         WRITE(ErrorMessage, 1008)  SeqToDateStringYMD(this%SDate);  CALL PassMsg
         WRITE(ErrorMessage, 1009)  SeqToDateStringYMD(this%EDate);  CALL PassMsg
         WRITE(ErrorMessage, 1010)  this%NumDays;         CALL PassMsg
         WRITE(ErrorMessage, 1011)  this%NumDataColumns;  CALL PassMsg

         1001 FORMAT('InfoIsValid = ', L1)
         1002 FORMAT('FormatType  = ', I0)
         1003 FORMAT('NumLines    = ', I0)
         1004 FORMAT('SubbasinNum = ', I0)
         1005 FORMAT('Elevation   = ', E10.4E2)
         1006 FORMAT('Area        = ', E10.4E2)
         1007 FORMAT('Bsn         = ', A)
         1008 FORMAT('SDate       = ', A10)
         1009 FORMAT('EDate       = ', A10)
         1010 FORMAT('NumDays     = ', I0)
         1011 FORMAT('NumDataCols = ', I0)
      END SUBROUTINE PrintHeaderInfo

      !--------------------------------------------------------
      SUBROUTINE ReadHeaderInfo(FileName, HdrInfo)
      IMPLICIT NONE
      CHARACTER(LEN=*),       INTENT(IN)    :: FileName
      TYPE (THeaderInfoType), INTENT(INOUT) :: HdrInfo
      
      INTEGER :: I, U1
      CHARACTER(LEN=300), DIMENSION(9) :: HLines

      !
      !  Initialize the HdrInfo structure.
      !  Note that nothing is done with the allocatable arrays. They 
      !  are just left unallocated for the moment. They will be allocated
      !  as they are needed and as we get the necessary info to size
      !  them correctly.
      !      
      CALL HdrInfo%Clear()
      HdrInfo%FormatType = HFT_SingleSubbasin
      HdrInfo%InfoIsValid = .TRUE.     ! assume ok by default.

      !
      !  Here I diverge from the standard GLSHFS code. In the original code from
      !  GLSHFS_Files this calls a procedure to read a generic header, and the
      !  procedure determines the type first.  Here I am going to straight up
      !  assume it matches a specific format.  So I will
      !  skip the parsing and just use the procedure to read that type of header.
      !  I need to read the 10 header lines into an array before calling it.
      !
      U1 = -1
      U1 = GetFreeUnitNumber();  IF (ErrorLevel .NE. 0) GOTO 899
      OPEN(UNIT=U1, FILE=TRIM(FileName), STATUS='OLD', ERR=811)
      CALL FileWasOpened(U1)
      DO I = 1, 9
         READ(U1, 1000, ERR=812) HLines(I) 
      END DO
      CLOSE(U1)
      CALL FileWasClosed(U1);  U1 = -1
      
      CALL ParseTheHeader(FileName, HLines, HdrInfo);  IF (ErrorLevel .NE. 0) GOTO 899
      RETURN

  811 ErrorMessage = 'Error opening input file: '//TRIM(FileName); CALL PassMsg; GOTO 898   
  812 ErrorMessage = 'Error reading input file: '//TRIM(FileName); CALL PassMsg; GOTO 898   
   
  898 ErrorLevel = 1
  899 ErrorMessage = '[traceback] : ReadHeaderInfo'; CALL PassMsg
      CALL HdrInfo%Clear()
      HdrInfo%InfoIsValid = .FALSE.
      CLOSE(U1, IOSTAT=I)
      IF (U1 .GT. 0) CALL FileWasClosed(U1)
      RETURN
      
 1000 FORMAT(A300)
      END SUBROUTINE ReadHeaderInfo

!------------------------------------------------------------------      
      !------------------------------------------------------------
      !  Parse the header section 
      !  This code is ALMOST the same as ParseSingleSubbasinHeader() from the
      !  standard GLSHFS source code, but it is simplified to ignore the
      !  contents of line 3. We might be doing a full-lake summary file or
      !  a single-subbasin summary file in this program. So that line may
      !  be different, and it's contents don't really matter for our
      !  purpose with this special conversion program.
      !------------------------------------------------------------
      SUBROUTINE ParseTheHeader(Filename, HLines, HdrInfo)
      IMPLICIT NONE
      CHARACTER(LEN=*),                 INTENT(IN)    :: FileName
      CHARACTER(LEN=300), DIMENSION(:), INTENT(IN)    :: HLines
      TYPE (THeaderInfoType),           INTENT(INOUT) :: HdrInfo
      
      INTEGER :: I, IOS, LkNum, Seq
      INTEGER :: NumStrings
      CHARACTER(LEN=100) :: S, TStr
      CHARACTER(LEN=100), DIMENSION(5000) :: CsvStrings

      !
      !  line 1
      !  Originally I checked this for matching a specific string, but now it is free text.
      !
      CALL ParseCommaSepLine(HLines(1), CsvStrings, NumStrings); IF (ErrorLevel .NE. 0) GOTO 899
      IF (NumStrings .LT. 1) GOTO 301
         
      !
      !  Line 2
      !
      CALL ParseCommaSepLine(HLines(2), CsvStrings, NumStrings); IF (ErrorLevel .NE. 0) GOTO 899
      IF (NumStrings .LT. 2) GOTO 302
      S = GetLowercase(StripAllBlanks(CsvStrings(1)))
      S = TRIM(StripDoubleQuotes(S))
      IF (TRIM(S) .NE. 'lake:') GOTO 332
      S = TRIM(ADJUSTL(CsvStrings(2)))
      LkNum = LakeNumberFromName3(S); IF (ErrorLevel .NE. 0) LkNum = -1
      ErrorLevel = 0
      IF (LkNum .LT. 1) THEN
         LkNum = LakeNumberFromName10(S); IF (ErrorLevel .NE. 0) LkNum = -1
         ErrorLevel = 0
      END IF
      IF (LkNum .LT. 1) GOTO 402
      HdrInfo%Bsn = LakeName3(LkNum)     ! save the 3-char lake name
      
      !
      !  Line 3
      !
      !  ***IGNORED***
            
      !
      !  Line 4
      !
      CALL ParseCommaSepLine(HLines(4), CsvStrings, NumStrings); IF (ErrorLevel .NE. 0) GOTO 899
      IF (NumStrings .LT. 2) GOTO 304
      S = GetLowercase(StripAllBlanks(CsvStrings(1)))
      S = TRIM(StripDoubleQuotes(S))
      IF (TRIM(S) .NE. 'area:') GOTO 334
      TStr = TRIM(ADJUSTL(CsvStrings(2)));  READ(TStr, *, ERR=404) HdrInfo%Area

      !
      !  Line 5
      !
      CALL ParseHdrLineDateField(TRIM(FileName), HLines(5), 5, 'Starts(ymd)', Seq); IF (ErrorLevel .NE. 0) GOTO 898
      HdrInfo%SDate = Seq
           
      !
      !  Line 6
      !
      CALL ParseHdrLineDateField(TRIM(FileName), HLines(6), 6, 'Ends(YMD)', Seq); IF (ErrorLevel .NE. 0) GOTO 898
      HdrInfo%EDate = Seq
            
      !
      !  Line 7 tracks the newest "new data" that has been added/updated.
      !  When GLSHFS updates LBRM output files using this data it will reset
      !  it to 9999-99-99 to indicate no new data.
      !
      !  ***IGNORED***

      !
      !  Line 8
      !      
      CALL ParseCommaSepLine(HLines(8), CsvStrings, NumStrings); IF (ErrorLevel .NE. 0) GOTO 899
      IF (NumStrings .LT. 3) GOTO 308
      S = GetLowercase(StripAllBlanks(CsvStrings(1)))
      S = TRIM(StripDoubleQuotes(S))
      IF (TRIM(S) .NE. 'date') GOTO 338
      HdrInfo%NumDataColumns = NumStrings - 1
      I = HdrInfo%NumDataColumns
      ALLOCATE(HdrInfo%DataType(I), HdrInfo%DataUnit(I), STAT=IOS)
      IF (IOS .NE. 0) GOTO 851
      DO I = 1, HdrInfo%NumDataColumns
         TStr = TRIM(ADJUSTL(CsvStrings(I+1)))
         TStr = TRIM(StripDoubleQuotes(TStr))
         HdrInfo%DataType(I) = GlerlDataTypeFromString(TStr)
         IF (HdrInfo%DataType(I) .EQ. GDT_Undefined) GOTO 408
      END DO

      !
      !  Line 9
      !
      CALL ParseCommaSepLine(HLines(9), CsvStrings, NumStrings); IF (ErrorLevel .NE. 0) GOTO 899
      IF (NumStrings .LT. 3) GOTO 309
      S = GetLowercase(StripAllBlanks(CsvStrings(1)))
      S = TRIM(StripDoubleQuotes(S))
      IF (TRIM(S) .NE. 'yyyy-mm-dd') GOTO 339
      I = NumStrings - 1
      IF (I .NE. HdrInfo%NumDataColumns) GOTO 409
      DO I = 1, HdrInfo%NumDataColumns
         TStr = TRIM(ADJUSTL(CsvStrings(I+1)))
         TStr = TRIM(StripDoubleQuotes(TStr))
         HdrInfo%DataUnit(I) = GlerlDataUnitFromString(TStr)
         IF (HdrInfo%DataUnit(I) .EQ. GDU_Undefined) GOTO 409
      END DO

      HdrInfo%InfoIsValid = .TRUE.

      RETURN
      
      !
      !  Errors messages
      !      
  301 ErrorMessage = 'Formatting error on line 1 (too few entries) in '//TRIM(FileName); CALL PassMsg; GOTO 898
  302 ErrorMessage = 'Formatting error on line 2 (too few entries) in '//TRIM(FileName); CALL PassMsg; GOTO 898
  304 ErrorMessage = 'Formatting error on line 4 (too few entries) in '//TRIM(FileName); CALL PassMsg; GOTO 898
  308 ErrorMessage = 'Formatting error on line 8 (too few entries) in '//TRIM(FileName); CALL PassMsg; GOTO 898
  309 ErrorMessage = 'Formatting error on line 9 (too few entries) in '//TRIM(FileName); CALL PassMsg; GOTO 898
   
  332 ErrorMessage = 'Error with label string on line 2 in '//TRIM(FileName); CALL PassMsg; GOTO 898
  334 ErrorMessage = 'Error with label string on line 4 in '//TRIM(FileName); CALL PassMsg; GOTO 898
  338 ErrorMessage = 'Error with label string on line 8 in '//TRIM(FileName); CALL PassMsg; GOTO 898
  339 ErrorMessage = 'Error with label string on line 9 in '//TRIM(FileName); CALL PassMsg; GOTO 898
   
  402 ErrorMessage = 'Invalid lake named on line 2 in '                 //TRIM(FileName); CALL PassMsg; GOTO 898
  404 ErrorMessage = 'Error parsing area on line 4 in '                 //TRIM(FileName); CALL PassMsg; GOTO 898
  408 ErrorMessage = 'Error parsing line 8 in '                         //TRIM(FileName); CALL PassMsg; GOTO 898
  409 ErrorMessage = 'Error parsing line 9 in '                         //TRIM(FileName); CALL PassMsg; GOTO 898
  
      !
      !  memory allocation error
      !      
  851 ErrorMessage = 'Error allocating memory for temporary storage of header information'; CALL PassMsg
      ErrorMessage = 'while reading '//TRIM(FileName); CALL PassMsg; GOTO 898

    
  898 ErrorLevel = 1
  899 ErrorMessage = '[traceback] : ParseTheHeader'; CALL PassMsg
      CALL HdrInfo%Clear()
      RETURN
      
      END SUBROUTINE ParseTheHeader
      
!------------------------------------------------------------------      
      !------------------------------------------------------------
      !  Common routines for parsing header section lines
      !------------------------------------------------------------
      SUBROUTINE ParseHdrLineDataType(FileName, Line, LineNum, DType)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN)  :: FileName, Line
      INTEGER,          INTENT(IN)  :: LineNum
      INTEGER,          INTENT(OUT) :: DType
      INTEGER :: NumStr
      CHARACTER(LEN=300) :: S, CsvStrings(100)
      
      CALL ParseCommaSepLine(Line, CsvStrings, NumStr); IF (ErrorLevel .NE. 0) GOTO 899
      IF (NumStr .LT. 2) GOTO 801
      S = GetLowercase(StripAllBlanks(CsvStrings(1)))
      S = TRIM(StripDoubleQuotes(S))
      IF (TRIM(S) .NE. 'datatype:') GOTO 802
      S = GetLowercase(TRIM(ADJUSTL(CsvStrings(2))))
      DType = GlerlDataTypeFromString(TRIM(S))
      IF (DType .EQ. GDT_Undefined) GOTO 803
      RETURN
      
  801 WRITE(ErrorMessage, 1001) LineNum, TRIM(FileName); CALL PassMsg; GOTO 898
  802 WRITE(ErrorMessage, 1002) LineNum, TRIM(FileName); CALL PassMsg; GOTO 898
  803 WRITE(ErrorMessage, 1003) LineNum, TRIM(FileName); CALL PassMsg; GOTO 898
  
  898 ErrorLevel = 1
  899 ErrorMessage = '[traceback] ParseHdrLineDataType';  CALL PassMsg
      RETURN
      
 1001 FORMAT('Formatting error on line ', I0, ' (too few entries) in ', A)
 1002 FORMAT('Error with label string on line ', I0, ' in ', A)
 1003 FORMAT('Error: invalid data type string on line ', I0, ' in ', A)
      END SUBROUTINE ParseHdrLineDataType

!------------------------------------------------------------------      
      !---------------------------------------------------------------------------
      SUBROUTINE ParseHdrLineDataUnit(FileName, Line, LineNum, DUnit)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN)  :: FileName, Line
      INTEGER,          INTENT(IN)  :: LineNum
      INTEGER,          INTENT(OUT) :: DUnit
      INTEGER :: NumStr
      CHARACTER(LEN=150) :: S, CsvStrings(100)
      
      CALL ParseCommaSepLine(Line, CsvStrings, NumStr)
      IF (NumStr .LT. 2) GOTO 801
      S = GetLowercase(StripAllBlanks(CsvStrings(1)))
      S = TRIM(StripDoubleQuotes(S))
      IF (TRIM(S) .NE. 'dataunits:') GOTO 802
      S = GetLowercase(TRIM(ADJUSTL(CsvStrings(2))))
      DUnit = GlerlDataUnitFromString(TRIM(S))
      IF (DUnit .EQ. GDU_Undefined) GOTO 803
      RETURN
      
  801 WRITE(ErrorMessage, 1001) LineNum, TRIM(FileName); CALL PassMsg; GOTO 898
  802 WRITE(ErrorMessage, 1002) LineNum, TRIM(FileName); CALL PassMsg; GOTO 898
  803 WRITE(ErrorMessage, 1003) LineNum, TRIM(FileName); CALL PassMsg; GOTO 898
  
  898 ErrorLevel = 1
      RETURN
      
 1001 FORMAT('Formatting error on line ', I0, ' (too few entries) in ', A)
 1002 FORMAT('Error with label string on line ', I0, ' in ', A)
 1003 FORMAT('Error: invalid data unit string on line ', I0, ' in ', A)
      END SUBROUTINE ParseHdrLineDataUnit

!------------------------------------------------------------------      
      !---------------------------------------------------------------------------
      SUBROUTINE ParseHdrLineDateField(FileName, Line, LineNum, IDStr, Seq)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN)  :: FileName, Line, IDStr
      INTEGER,          INTENT(IN)  :: LineNum
      INTEGER,          INTENT(OUT) :: Seq
      INTEGER :: NumStr
      LOGICAL :: F
      CHARACTER(LEN=150) :: SF, ST, STC, CsvStrings(100)
      
      CALL ParseCommaSepLine(Line, CsvStrings, NumStr)
      IF (NumStr .LT. 2) GOTO 801
      SF = GetLowercase(StripAllBlanks(CsvStrings(1)))
      SF = TRIM(StripDoubleQuotes(SF))                  ! string from the file
      ST  = GetLowercase(StripAllBlanks(IDStr))
      ST  = TRIM(StripDoubleQuotes(ST))                 ! target string (no colon) for comparison
      STC = TRIM(ST) // ':'                             ! target string (with colon) for comparison
      F = .FALSE.
      IF (TRIM(SF) .EQ. TRIM(ST))  F = .TRUE.
      IF (TRIM(SF) .EQ. TRIM(STC)) F = .TRUE. 
      IF (.NOT. F) GOTO 802
      Seq = DateStringYMDToSeq(TRIM(ADJUSTL(CsvStrings(2)))); IF (ErrorLevel .NE. 0) GOTO 803
      RETURN
      
  801 WRITE(ErrorMessage, 1001) LineNum, TRIM(FileName); CALL PassMsg; GOTO 898
  802 WRITE(ErrorMessage, 1002) LineNum, TRIM(FileName); CALL PassMsg; GOTO 898
  803 WRITE(ErrorMessage, 1003) LineNum, TRIM(FileName); CALL PassMsg; GOTO 898
  
  898 ErrorLevel = 1
      Seq = MissingData_Date
      RETURN
      
 1001 FORMAT('Formatting error on line ', I0, ' (too few entries) in ', A)
 1002 FORMAT('Error with label string on line ', I0, ' in ', A)
 1003 FORMAT('Error with date string on line ', I0, ' in ', A)
      END SUBROUTINE ParseHdrLineDateField
      

!------------------------------------------------------------------      
      !------------------------------------------------------------
      !  Read the data value section of any CSV file. They are all the same format.
      !  Interpretation of the meaning for each data column will be up to the
      !  calling procedure.
      !
      !  The Data2D array is assumed to already be allocated/sized with
      !  the first index being used for the days (1..numdays) and the second
      !  index being the number of stations, datatypes, or whatever.
      !
      !  The data is read as 1 column of date info (Y-M-D), which is then
      !  converted into a sequence number and assigned into the DateSeqs array.
      !  The rest of the columns will be assigned into the Data2D array, which
      !  is indexed (1..maxrow, 1..maxcol).
      !
      !  Also assumed is that the file pointer is positioned at the correct line,
      !  and we can just start reading data lines (without skipping header lines).
      !------------------------------------------------------------
      SUBROUTINE ReadDataSectionCSV(FileName, U1, NumHdrLines, SSeq, ESeq, Data2D)
      IMPLICIT NONE
      CHARACTER(LEN=*),     INTENT(IN)    :: FileName
      INTEGER,              INTENT(IN)    :: U1, NumHdrLines, SSeq, ESeq
      REAL, DIMENSION(:,:), INTENT(INOUT) :: Data2D
      
      INTEGER :: J, Row, Col, LineNum, IOS
      INTEGER :: MaxRow, MaxCol, NumStrings
      INTEGER :: Seq
      REAL    :: DVal
      CHARACTER(LEN=100) :: TStr
      CHARACTER(LEN=500) :: Line
      CHARACTER(LEN=500), DIMENSION(50) :: CsvStrings
      
      MaxRow = UBOUND(Data2D, 1)       ! number of days
      MaxCol = UBOUND(Data2D, 2)       ! number of data types, stations, etc
      
      LineNum = NumHdrLines      
      READ(U1, 1000, IOSTAT=IOS) Line
      DO WHILE (IOS .EQ. 0)
         LineNum = LineNum + 1
         CALL ParseCommaSepLine(Line, CsvStrings, NumStrings)
         IF (NumStrings .LT. 2) GOTO 401
         IF (NumStrings .GT. MaxCol+1) GOTO 402
         Seq = DateStringYMDToSeq(TRIM(ADJUSTL(CsvStrings(1))))
         IF ((Seq .GE. SSeq) .AND. (Seq .LE. ESeq)) THEN
            Row = Seq - SSeq + 1
            IF (Row .LE. MaxRow) THEN
               DO Col = 1, NumStrings-1
                  TStr = TRIM(ADJUSTL(CsvStrings(Col+1))); READ(TStr, *, IOSTAT=J) DVal
                  IF (J .NE. 0) DVal = MissingData_Real
                  Data2D(Row,Col) = DVal
               END DO
            ELSE
               WRITE(ErrorMessage, 5001) TRIM(Filename), Row, MaxRow, Seq , SSeq
               CALL PassMsg
               GOTO 898
            END IF
         END IF
         READ(U1, 1000, IOSTAT=IOS) Line
      END DO
      RETURN
      
      !
      !  Error messages
      !      
  401 WRITE(ErrorMessage, 4001) LineNum, TRIM(FileName); CALL PassMsg; GOTO 898
  402 WRITE(ErrorMessage, 4002) LineNum, TRIM(FileName); CALL PassMsg; GOTO 898
  
  
  898 ErrorLevel = 1
      ErrorMessage = '[traceback] : ReadDataSectionCSV'; CALL PassMsg
      RETURN
      
 1000 FORMAT(A500)
 4001 FORMAT('Formatting error on line ', I0, 'of file ', A, '; too few columns.')
 4002 FORMAT('Formatting error on line ', I0, 'of file ', A, '; too many columns.')
 5001 FORMAT('Yikes... File ', A, ';  Row=', I0, ' Max=', I0, ' Seq=', I0, ' SSeq=', I0)
 
      END SUBROUTINE ReadDataSectionCSV
      
      !------------------------------------------------------------------
      !  Given a 3-character lake name, retrieve the lake number for that lake.
      !------------------------------------------------------------------
      INTEGER FUNCTION LakeNumberFromName3(Bsn)
      IMPLICIT  NONE
      CHARACTER (LEN=*), INTENT(IN) :: Bsn

      INTEGER :: I
      CHARACTER (LEN=3)  :: S3

      LakeNumberFromName3 = -1       ! default bad value

      S3 = 'xxx'
      IF (LEN(Bsn) .GE. 3) S3 = ADJUSTL(Bsn(1:3))
      IF (S3 .EQ. 'xxx') RETURN

      CALL LowerCase(S3)
      DO I = 1, 7
         IF (S3 .EQ. LakeName3(I)) LakeNumberFromName3 = I
      END DO
      RETURN
     
      END FUNCTION LakeNumberFromName3

      !------------------------------------------------------------------
      !  Given a 10-character lake name, retrieve the lake number for that lake.
      !------------------------------------------------------------------
      INTEGER FUNCTION LakeNumberFromName10(LkName)
      IMPLICIT  NONE
      CHARACTER (LEN=*), INTENT(IN) :: LkName

      INTEGER :: I
      CHARACTER (LEN=10)  :: S1, S2

      LakeNumberFromName10 = -1       ! default bad value

      S1 = 'xxxxxxxxxx'
      IF (LEN_TRIM(LkName) .GT. 10) RETURN
      S1 = TRIM(ADJUSTL(LkName))
      IF (S1 .EQ. 'xxxxxxxxxx') RETURN

      CALL LowerCase(S1)
      DO I = 1, 7
         S2 = LakeName10(I)
         CALL LowerCase(S2)
         IF (TRIM(S1) .EQ. TRIM(S2)) LakeNumberFromName10 = I
      END DO
      RETURN
     
      END FUNCTION LakeNumberFromName10



END MODULE glsummary2mon_subs

!==============================================================================
!==============================================================================
!==============================================================================
      PROGRAM glshfs_summary_to_mon
      USE GLSHFS_Util
      USE GlerlDataTypesAndUnits
      USE glsummary2mon_subs
      IMPLICIT NONE

      INTEGER, PARAMETER :: NumDataTypes = 34
      
      INTEGER :: IOS, I, DT, Cnt, M1, M12
      INTEGER :: D, M, Y, SD, SM, SY, ED, EM, EY, NumYears
      INTEGER :: Seq, SSeq, ESeq, NumMons, NumDays, NumArgs
      INTEGER :: UserStartYear, UserEndYear

      REAL :: MonTot, DatVal
      REAL, DIMENSION(:,:), ALLOCATABLE :: DataIn, MonDat

      CHARACTER (LEN=40)  :: FileIn, FileOut, S
      CHARACTER (LEN=80)  :: CmdLine
      CHARACTER (LEN=10), DIMENSION(NumDataTypes) :: OutVals
      CHARACTER (LEN=45), DIMENSION(5) ::  ArgStrings

      CHARACTER (LEN=10), DIMENSION(NumDataTypes), PARAMETER :: DT_Name =  (/      &
         'OvrLkRunof', 'OvrLkPrecp', ' Evaporatn', 'NetBsnSupl', 'OvrLdRunof',     &
         'OvrLdPrecp', 'OvrLkATMin', 'OvrLkATMax', 'OvrLkATAvg', 'OvrLkDewpt',     &
         'OvrLkWndSp', 'OvrLkCloud', 'OvrLdATMin', 'OvrLdATMax', 'OvrLdATAvg',     &
         'OvrLdDewpt', 'OvrLdWndSp', 'OvrLdCloud', 'UprSoilMst', 'LwrSoilMst',     &
         'GroundWatr', 'SurfaceSto', 'SnowWatrEq', ' WaterTemp', '   IceArea',     &
         '  IceDepth', '   IceTemp', 'ReflectRad', ' LatentRad', 'SensiblRad',     &
         ' Advection', 'IncidntRad', '  NetLWRad', ' TotalHeat' /)
      CHARACTER (LEN=10), DIMENSION(NumDataTypes), PARAMETER :: DT_Units = (/      &
         'Millimeter', 'Millimeter', 'Millimeter', 'Millimeter', 'Millimeter',     &
         'Millimeter', '   Celsius', '   Celsius', '   Celsius', '   Celsius',     &
         'Meters/Sec', '   Percent', '   Celsius', '   Celsius', '   Celsius',     &
         '   Celsius', 'Meters/Sec', '   Percent', 'Millimeter', 'Millimeter',     &
         'Millimeter', 'Millimeter', 'Millimeter', '   Celsius', '   Percent',     &
         'Millimeter', '   Celsius', ' Watts/SqM', ' Watts/SqM', ' Watts/SqM',     &
         ' Watts/SqM', ' Watts/SqM', ' Watts/SqM', '  Calories' /)
       
      !  1 = Monthly value is TOTAL of daily values
      !  2 = Monthly value is MEAN of daily values
      INTEGER, DIMENSION(NumDataTypes), PARAMETER :: DT_MonthlyMethod = (/         &
                              1, 1, 1, 1, 1, 1, 2, 2, 2, 2,     &
                              2, 2, 2, 2, 2, 2, 2, 2, 2, 2,     &
                              2, 2, 2, 2, 2, 2, 2, 2, 2, 2,     &
                              2, 2, 2, 2 /)
       
      TYPE (THeaderInfoType) :: HdrInfo

      ErrorLevel = 0
      ErrorMessage = ' '
      CALL ClearFilesOpen()

      CALL GET_COMMAND(CmdLine)
      CALL ParseCmdLine(CmdLine, ArgStrings, NumArgs)
      IF (NumArgs .LT. 3) GOTO 875

      FileIn  = TRIM(ArgStrings(2))
      FileOut = TRIM(ADJUSTL(ArgStrings(3)))

      UserStartYear = -9999
      UserEndYear   = -9999
      IF (NumArgs .GE. 5) THEN
         S = TRIM(ArgStrings(4))
         READ(S, *, IOSTAT=IOS) Y
         IF (IOS .EQ. 0) THEN
            IF ((Y .GT. 1500) .AND. (Y .LT. 2500)) UserStartYear = Y
         END IF
         S = TRIM(ArgStrings(5))
         READ(S, *, IOSTAT=IOS) Y
         IF (IOS .EQ. 0) THEN
            IF ((Y .GT. 1500) .AND. (Y .LT. 2500)) UserEndYear = Y
         END IF
      END IF

      !
      !  First just read the header info.  The routine for this is
      !  grabbed from the source code for GLSHFS.  It will
      !  open and close the file, so I just pass it the file name.
      !  It will return the header info in a THeaderInfoType structure.
      !
      CALL ReadHeaderInfo(FileIn, HdrInfo); IF (ErrorLevel .NE. 0) GOTO 899
      SSeq = HdrInfo%SDate
      ESeq = HdrInfo%EDate

      !
      !  Adjust period of interest to match user spec, if given 
      !
      IF (UserStartYear .GT. -999) THEN
         CALL DateSequence( 1,  1, UserStartYear, Seq)
         IF (Seq .GT. SSeq) SSeq = Seq
      END IF
      IF (UserEndYear .GT. -999) THEN
         CALL DateSequence(31, 12, UserEndYear, Seq) 
         IF (Seq .LT. ESeq) ESeq = Seq
      END IF

      
      !
      !  Allocate arrays for the data values
      !  Note that the MonDat() array starts at January of the start 
      !  year, regardless of the actual data start.  And it goes to
      !  December of the end year.
      !
      CALL SequenceDate(SD, SM, SY, SSeq); IF (ErrorLevel .NE. 0) GOTO 899
      CALL SequenceDate(ED, EM, EY, ESeq); IF (ErrorLevel .NE. 0) GOTO 899
      NumYears = EY-SY+1
      NumMons = NumYears*12
      NumDays = ESeq-SSeq+1
      ALLOCATE(MonDat(NumMons,NumDataTypes),            &
               DataIn(NumDays,NumDataTypes), STAT=IOS)
         
      !
      !  The routine that reads the data lines is also being pulled from 
      !  the GLSHFS source code, but in this case, the routine assumes that
      !  the file is already open and that the file header lines have 
      !  already been read (or skipped). So I need to prep things that way
      !  before calling the routine.
      !
      OPEN(UNIT=11, FILE=TRIM(FileIn), STATUS='OLD', ERR=811)
      CALL FileWasOpened(11)
      DO I = 1, 9
         READ(11, *, ERR=812)   ! skip the 9 header lines
      END DO
      CALL ReadDataSectionCSV(FileIn, 11, 9, SSeq, ESeq, DataIn)

      !
      !  Compute monthly values from the daily values
      !
      DO DT = 1, NumDataTypes
         MonDat(:,DT) = -999.99
         MonTot = 0.0
         Cnt = 0
         DO Seq = SSeq, ESeq
            CALL SequenceDate(D, M, Y, Seq); IF (ErrorLevel .NE. 0) GOTO 899
            DatVal = DataIn(Seq-SSeq+1, DT)
            IF (DatVal .GT. -998.) THEN
               MonTot = MonTot + DatVal
               Cnt = Cnt + 1
            END IF
            IF (IsLastDayOfMonth(D, M, Y) .OR. (Seq .EQ. ESeq)) THEN
               I = (Y-SY)*12 + M
               IF (Cnt .GT. 0) THEN
                  IF (DT_MonthlyMethod(DT) .EQ. 1) THEN
                     MonDat(I,DT) = MonTot
                  ELSE IF (DT_MonthlyMethod(DT) .EQ. 2) THEN
                     MonDat(I,DT) = MonTot / Cnt
                  END IF
               END IF
               MonTot = 0
               Cnt = 0
            END IF
         END DO
      END DO
         
      !
      !  Write output file
      !
      OPEN(UNIT=21, FILE=TRIM(FileOut), STATUS='REPLACE', ERR=821)
      CALL FileWasOpened(21)
      PRINT*, 'Generating ' // TRIM(FileOut)
      WRITE(21, 2101, ERR=823) TRIM(FileIn)
      CALL SequenceDate(SD, SM, SY, SSeq)
      CALL SequenceDate(ED, EM, EY, ESeq)
      WRITE(21, 2102, ERR=823) SY, SM, SD
      WRITE(21, 2103, ERR=823) EY, EM, ED
      WRITE(21, 2104, ERR=823) (DT_Name(I),  I=1,34)
      WRITE(21, 2105, ERR=823) (DT_Units(I), I=1,34)
      DO Y = SY, EY
         M1  = 1
         M12 = 12
         IF (Y .EQ. SY) M1  = SM
         IF (Y .EQ. EY) M12 = EM
         DO M = M1, M12
            I = (Y-SY)*12 + M
            DO DT = 1, NumDataTypes
               IF (MonDat(I,DT) .GT. -999.8) THEN
                  IF (DT .LT. NumDataTypes) THEN
                     WRITE(OutVals(DT), '(F10.3)') MonDat(I,DT)
                  ELSE
                     WRITE(OutVals(DT), '(E10.3E2)') MonDat(I,DT)
                  END IF
               ELSE
                  OutVals(DT) = '        NA'
               END IF
            END DO
            WRITE(21, 2110, ERR=823) Y,M,(OutVals(DT), DT=1,NumDataTypes)
         END DO
      END DO
      CLOSE(21)
      CALL FileWasClosed(21)
      
      DEALLOCATE(MonDat, DataIn, STAT=IOS)

      GOTO 999

!
!  Handlers for I/O errors
!
 811  ErrorMessage = 'Error opening file ' // TRIM(FileIn)
      CALL PassMsg; GOTO 898
 812  ErrorMessage = 'Error reading file ' // TRIM(FileIn)
      CALL PassMsg; GOTO 898
 821  ErrorMessage = 'Error opening file ' // TRIM(FileOut)
      CALL PassMsg; GOTO 898
 823  ErrorMessage = 'Error writing file ' // TRIM(FileOut)
      CALL PassMsg; GOTO 898

 875  PRINT*, 'USAGE: glshfs_summary2mon inputfile outputfile [StartYear EndYear]'
      PRINT*, '       e.g.: glshfs_summary2mon summary_sup.csv summary_sup_mon.csv'
      CALL EXIT(1)
      
 898  ErrorLevel = 1
 899  ErrorMessage = '[traceback] : glshfs_summary2mon...'
      CALL PassMsg
      GOTO 999


!
!  Final cleanup
!
      DEALLOCATE(MonDat, DataIn, STAT=IOS)
  999 CALL CloseAllFiles()

!
!  FORMAT statements
!
 2101 FORMAT('Monthly data from ', A)
 2102 FORMAT('Starts(YMD):,', I4, 2('-', I2.2))
 2103 FORMAT('Ends(YMD):,  ', I4, 2('-', I2.2))
 2104 FORMAT('Date   ', 34(',', A10))
 2105 FORMAT('YYYY-MM', 34(',', A10))
 2110 FORMAT(I4, '-', I2.2, 34(',', A10))

      END PROGRAM


