!---------------------------------------------------------------------------------
!  This program trims a subdata_*.csv file, removing any extraneous
!  missing data at end of the file, then updating the header info.
!  Written by Tim Hunter Sep 16, 2020.
! 
!  input file(s):  subdata_xxxnn.csv or
!                  subdata_xxx_aaaa.csv
! 
!  where xxx  = 3-character lake code (sup, mic, hur, etc)
!        nn   = 2-digit subbasin number (00, 01, 02, ...)
!        aaaa = area code (land, basn, lake)
!
!  Note that I am utilizing the code from GLSHFS for reading the GLSHFS files.
!  That requires a little extra overhead, but saves me from "reinventing the wheel".
!  I am also grabbing various other bits and pieces from the GLSHFS code, sometimes
!  with small modifications for this program.  It keeps things relatively 
!  stand-alone, but without having to do *everything* from scratch.
!
!---------------------------------------------------------------------------------

!==============================================================================
!==============================================================================
!==============================================================================
      PROGRAM glshfs_trim_metfiles
      USE GL_Constants
      USE MyKinds                     ! part of glshfs_util.f90
      USE GLSHFS_Util
      USE GLSHFS_Files
      USE DailyDataStructures
      USE GlerlDataTypesAndUnits
      USE DailyDataCollections
      IMPLICIT NONE

      INTEGER :: IOS, I, K, ArgC, DT, NumDays, NumStns, NumSubs
      INTEGER :: Sub, Sub1, Sub2, LkNum, DTNum, SNum, DUnits
      INTEGER :: D, M, Y, SD, SM, SY, ED, EM, EY
      INTEGER :: Seq, SSeq, ESeq, NewESeq, Stn_SSeq, Stn_ESeq
      LOGICAL :: OK, Found
      REAL, DIMENSION(7) :: Raw
      REAL, DIMENSION(:), ALLOCATABLE  :: OneStnData

      CHARACTER (LEN=3)   :: Bsn
      CHARACTER (LEN=15)  :: DtStr
      CHARACTER (LEN=80)  :: OldFile, NewFile, S
      CHARACTER (LEN=80)  :: CLine
      CHARACTER (LEN=45), DIMENSION(5)  :: ArgS

      INTEGER, DIMENSION(7), PARAMETER :: DataTypesToProcess = (/                  &
            GDT_AirtempMin, GDT_AirtempMax, GDT_Precipitation,                     &
            GDT_AirtempMean, GDT_DewpointMean, GDT_WindSpeed, GDT_CloudCover /)

      TYPE (TDlyData), POINTER   :: TDDP1, TDDP2, TDDP3, TDDP4, TDDP5, TDDP6, TDDP7
      TYPE (TDlyDataForSubbasin) :: SubData

      TYPE (TDlyData)                :: TDD, NewTDD 
      TYPE (TDlyData),       POINTER :: TDDP
      TYPE (TDlyDataMetStn)          :: NewStn
      TYPE (TDlyDataMetStn), POINTER :: TDDMSP      
      TYPE (TDlyDataForMultipleStations) :: StnData, NewData
      TYPE (THeaderInfoType)         :: OldHdrInfo, NewHdrInfo

      !
      ErrorLevel = 0
      ErrorMessage = ''
      CALL ClearFilesOpen()
      SubData = TDlyDataForSubbasin()

      !
      !  Get input from user [lake subbasin]
      !  GET_COMMAND() counts program name as the first argument
      !
      CALL GET_COMMAND(CLine)
      CALL ParseCmdLine(CLine, Args, ArgC)      
      IF (ArgC .LT. 2) GOTO 875
      
      !
      Bsn = TRIM(Args(2))
      LkNum = LakeNumberFromName3(Bsn)
      IF (LkNum .LT. 1) THEN
         PRINT*, 'Error with lake name. ('//TRIM(Bsn)//')'
         GOTO 875
      END IF
      CALL Lowercase(Bsn)
      LkNum = LakeNumberFromName3(Bsn)
      NumSubs = NumSubbasins(LkNum)
            
      !
      !  Main loop for stndata_*.csv files
      !    Use ReadFile_OneDataTypeManyStations()
      !    and WriteFile_OneDataTypeManyStations()
      !
      DO DT = 1, 7
         DTNum = DataTypesToProcess(DT)
         DTStr = GetLowercase(GlerlDataTypeString(DTNum))

         WRITE(OldFile, 4101) TRIM(DTStr), TRIM(Bsn)   
         WRITE(NewFile, 4102) TRIM(DTStr), TRIM(Bsn)   
         PRINT*, 'Trimming ' // TRIM(OldFile)
         PRINT*, '   Reading old data file'
         CALL ReadFile_OneDataTypeManyStations(TRIM(OldFile), StnData);  IF (ErrorLevel .NE. 0) GOTO 899
         SSeq = StnData%EarliestData
         ESeq = StnData%LatestData
         NumStns = StnData%NumStations

         NumDays = ESeq - SSeq + 1
!    print*, 'NumStns, NumDays=', NumStns, NumDays
!    print*, 'Old date extent = ', SeqToDateStringYMD(SSeq), ' - ', SeqToDateStringYMD(ESeq)

         !
         !  Read the contents of each station and find out the latest date of
         !  valid data.  
         !
         PRINT*, '   Finding valid data extent'
         ALLOCATE(OneStnData(NumDays), STAT=IOS)       ! temporary data storage     
         NewESeq = SSeq     
         Seq     = ESeq
         DO SNum = 1, NumStns
            TDDMSP => StnData%GetStationPtrByIndex(SNum)
            TDDP   => TDDMSP%GetPointerToDataOfType(DTNum)
            IF (ASSOCIATED(TDDP)) THEN
               Stn_SSeq = TDDP%GetStartDate()
               Stn_ESeq = TDDP%GetEndDate()
               OneStnData(:) = TDDP%GetDataVals()               ! the entire time-series for this station
               Found = .FALSE.
               Seq = Stn_ESeq
               DO WHILE ((.NOT. Found) .AND. (Seq .GT. NewESeq))
                  I = Seq - Stn_SSeq + 1
                  IF (I .GT. 0) THEN
                      IF (OneStnData(I) .GT. MissingData_Real_Test) THEN
                         Found = .TRUE.
                         IF (Seq .GT. NewESeq) NewESeq = Seq
                      END IF
                  END IF
                  Seq = Seq - 1
               END DO
            END IF
         END DO
         DEALLOCATE(OneStnData, STAT=IOS)
!    print*, 'New date extent = ', SeqToDateStringYMD(SSeq), ' - ', SeqToDateStringYMD(NewESeq)

         !
         !  I need the data units specifier for the write routine.  It's the
         !  same for all stations, so just get it from station 1.
         !
         TDDMSP => StnData%GetStationPtrByIndex(1)
         TDDP   => TDDMSP%GetPointerToDataOfType(DTNum)
         DUnits = TDDP%GetDataUnit()
         
         !
         !  Set up the NewData object that will be a trimmed version of the
         !  data read from the old file.
         !
         PRINT*, '   Building the trimmed data object'
         NewData = TDlyDataForMultipleStations(); IF (ErrorLevel .NE. 0) GOTO 899
         NewStn = TDlyDataMetStn()     ! data for one station (potentially multiple data types)
         NewTDD = TDlyData()           ! a single daily data timeseries (one data type)

         DO SNum = 1, NumStns
            TDDMSP => StnData%GetStationPtrByIndex(SNum)     ! a single station
            TDDP   => TDDMSP%GetPointerToDataOfType(DTNum)   ! a single data type for that station
            IF (ASSOCIATED(TDDP)) THEN         
               CALL NewTDD%Clear()
               OK = NewTDD%CopyFrom(TDDP)                    ! copy everything (meta and data)
               IF (.NOT. OK) THEN
                   PRINT*, 'Error copying single dataset'
                   GOTO 898
               END IF
                  
               CALL NewTDD%AdjustPeriod(SSeq, NewESeq)       ! trim the dataset
               CALL NewStn%Clear()
               OK = NewStn%CopyMetaFrom(TDDMSP)
               IF (.NOT. OK) THEN
                   PRINT*, 'Error copying single station metadata'
                   GOTO 898
               END IF

               OK = NewStn%AddDataset(NewTDD)                ! add the trimmed dataset to the station object
               IF (.NOT. OK) THEN
                   PRINT*, 'Error adding to single station'
                   GOTO 898
               END IF
               
               OK = NewData%AddStation(NewStn)               ! add station to the big object
               IF (.NOT. OK) THEN
                   PRINT*, 'Error adding station to NewData'
                   GOTO 898
               END IF
            END IF
         END DO               

         !
         !  Write the new file.
         !  First write it as a temporary file.
         !  Then replace the old file
         !
!    print*, 'NewData range=', SeqToDateStringYMD(NewData%EarliestData), ' - ', SeqToDateStringYMD(NewData%LatestData)
!    print*, 'New #stations=', NewData%NumStations
         
         PRINT*, '   Writing new data file'
         CALL WriteFile_OneDataTypeManyStations(TRIM(NewFile), NewData, DTNum, DUnits);  IF (ErrorLevel .NE. 0) GOTO 899
         CALL RenameFile(NewFile, OldFile, OK)
         
         CALL NewTDD%Clear()
         CALL NewStn%Clear()
         CALL NewData%Clear()
      END DO         

      !
      !  Main loop for subdata_*.csv files
      !    Use ReadFile_OneSubbasin()
      !    and WriteFile_OneSubbasin()
      !
      DO Sub = 0, NumSubs
         WRITE(OldFile,   4001) Bsn, Sub
         WRITE(NewFile, 4002) Bsn, Sub
         PRINT*, 'Trimming ' // TRIM(OldFile)
         INQUIRE(FILE=TRIM(OldFile), EXIST=OK)
         IF (.NOT. OK) THEN
            PRINT*, 'Missing file (', TRIM(OldFile), ')'
            PRINT*, 'Unable to perform conversion'
            GOTO 898
         END IF
         
         CALL ReadFile_OneSubbasin(TRIM(OldFile), SubData);  IF (ErrorLevel .NE. 0) GOTO 899
         SSeq = SubData%GetStartDate()
         ESeq = SubData%GetEndDate()
         IF (ESeq .LT. SSeq) THEN
            PRINT*, 'Error: End date is earlier than start date.'
            GOTO 898
         END IF

         !
         !  Get pointers to data objects
         !
         TDDP1 => SubData%GetPointerToDataOfType(GDT_AirtempMin)
         TDDP2 => SubData%GetPointerToDataOfType(GDT_AirtempMax)
         TDDP3 => SubData%GetPointerToDataOfType(GDT_Precipitation)
         TDDP4 => SubData%GetPointerToDataOfType(GDT_AirtempMean)
         TDDP5 => SubData%GetPointerToDataOfType(GDT_DewpointMean)
         TDDP6 => SubData%GetPointerToDataOfType(GDT_WindSpeed)
         TDDP7 => SubData%GetPointerToDataOfType(GDT_CloudCover)

         !
         !  Determine when the data ends.
         !  ANY data type qualifies.
         !
         NewESeq = ESeq
         Found = .FALSE.
         Seq = ESeq
         DO WHILE (.NOT. Found)
            I = Seq - SSeq + 1
            Raw(1) = TDDP1%GetDataVal(I)
            Raw(2) = TDDP2%GetDataVal(I)
            Raw(3) = TDDP3%GetDataVal(I)
            Raw(4) = TDDP4%GetDataVal(I)
            Raw(5) = TDDP5%GetDataVal(I)
            Raw(6) = TDDP6%GetDataVal(I)
            Raw(7) = TDDP7%GetDataVal(I)
            DO K = 1, 7
               IF (Raw(K) .GT. MissingData_Real_Test) THEN
                   Found = .TRUE.
               END IF
            END DO
            IF (Found) NewESeq = Seq
            Seq = Seq - 1
         END DO

         !
         !  Write the new file.
         !  First write it as a temporary file.
         !  Then replace the old file
         !
         CALL WriteFile_OneSubbasin(NewFile, SubData, SSeq, NewESeq)
         CALL RenameFile(NewFile, OldFile, OK)
      END DO         

      
      GOTO 999



 875  PRINT*, 'USAGE: glshfs_trim_metfiles  bsn'
      PRINT*, '       bsn = sup, mic, hur, etc'
      CALL EXIT(1)

      
 898  ErrorLevel = 1
 899  ErrorMessage = '[traceback] : glshfs_trim_metfiles [main]...'
      CALL PassMsg
      GOTO 999


!
!  Final cleanup
!
  999 CALL CloseAllFiles()

!
!  FORMAT statements
!

 4001 FORMAT('subdata_', A3, I2.2,  '.csv')
 4002 FORMAT('subdata_', A3, I2.2,  '.tmp')

 4101 FORMAT('stndata_', A, '_', A3, '.csv')
 4102 FORMAT('stndata_', A, '_', A3, '.tmp')
 
      END PROGRAM


