!============================================================================
!  This program reads the data from the subbasin meteorology files produced
!  by/for GLSHFS. It then computes a single weighted value (per day) and
!  outputs a weighted file of overLAND or overBASIN values. 
!
!  Input files are subdata_xxxnn.csv  (e.g. subdata_sup01.csv)
!  Output file is overland_met_xxx.csv or overbasin_met_xxx.csv
!----------------------------------------------------------------------------
MODULE LumpSubMet_Subs
      USE Glshfs_Util

      CHARACTER (LEN=3), PARAMETER :: LakeName3(7) =           &
         (/'sup', 'mic', 'hur', 'geo', 'stc', 'eri', 'ont'/)
      INTEGER, PARAMETER :: NumSubbasins(7) =                  &
         (/22, 27, 16, 13, 7, 21, 15/)

      INTEGER, PARAMETER :: MaxDays = 73200      ! 200 years
      
      
CONTAINS

      !------------------------------------------------------------------
      !  Given a 3-character lake name, retrieve the lake number for that lake.
      !------------------------------------------------------------------
      INTEGER FUNCTION LakeNumberFromName3(Bsn)
      IMPLICIT  NONE
      CHARACTER (LEN=*), INTENT(IN) :: Bsn

      INTEGER :: I
      CHARACTER (LEN=3)  :: S3

      LakeNumberFromName3 = -1       ! default bad value

      S3 = 'xxx'
      IF (LEN(Bsn) .GE. 3) S3 = ADJUSTL(Bsn(1:3))
      IF (S3 .EQ. 'xxx') RETURN

      CALL LowerCase(S3)
      DO I = 1, 7
         IF (S3 .EQ. LakeName3(I)) LakeNumberFromName3 = I
      END DO
      RETURN
     
      END FUNCTION LakeNumberFromName3


      !------------------------------------------------------------------
      !  Compute lumped values
      !------------------------------------------------------------------
      SUBROUTINE ComputeLumpedVals(Bsn, BasnLand)
      IMPLICIT NONE
      CHARACTER (LEN=*), INTENT(IN) :: Bsn          ! 'SUP', 'MIC', etc
      CHARACTER (LEN=*), INTENT(IN) :: BasnLand     ! 'BASN' or 'LAND'

      INTEGER :: I, J, S, IOS, Sub, NumStrings
      INTEGER :: Sub1, Sub2, NumSubs, LkNum
      INTEGER :: Seq, SSeq, ESeq, LSSeq, LESeq, NumDays
      REAL    :: DV, TotA, TotV, TotalArea
      REAL, DIMENSION(0:30) :: SubArea
      REAL, DIMENSION(:,:,:), ALLOCATABLE :: SubData
      REAL, DIMENSION(:,:),   ALLOCATABLE :: LumpData
      CHARACTER(LEN=4)  :: BL
      CHARACTER(LEN=5)  :: AreaOut
      CHARACTER(LEN=8)  :: OutStr(7)
      CHARACTER(LEN=10) :: SDate, EDate
      CHARACTER(LEN=50) :: AreaName
      CHARACTER(LEN=99) :: SubFile, OutFile
      CHARACTER(LEN=150) :: Line, Strings(10)
      

      !
      !  Verify and use arguments passed to routine
      !
      LkNum = LakeNumberFromName3(Bsn)
      IF (LkNum .LT. 1) THEN
         PRINT*, 'Invalid lake name ('//TRIM(Bsn)//') in ComputeLumpedVals.'
         GOTO 898
      END IF
      NumSubs = NumSubbasins(LkNum)
      
      BL = GetLowercase(BasnLand)
      IF (TRIM(BL) .EQ. 'land') THEN
         Sub1 = 1
         Sub2 = NumSubs
         AreaOut = 'land'
         AreaName = 'combined land areas of the ' // TRIM(Bsn) //' basin.'
         WRITE(OutFile, 1002) LakeName3(LkNum), 'land'
      ELSE IF (TRIM(BL) .EQ. 'basn') THEN
         Sub1 = 0
         Sub2 = NumSubs
         AreaOut = 'basin'
         AreaName = 'combined lake & land areas of the ' // TRIM(Bsn) //' basin.'
         WRITE(OutFile, 1002) LakeName3(LkNum), 'basn'
      ELSE
         PRINT*, 'Error in BasnLand parameter ('//TRIM(BasnLand)//').'
         PRINT*, 'Must be either "basn" or "land"'
         GOTO 898
      ENDIF

      !
      !  Read just the start/end dates from the subdata_xxxnn.csv files in
      !  order to determine the overall start/end of the available data and
      !  also the total area.
      !
      PRINT*, 'Determining overall period of record and total area'
      LSSeq = DateSeq_MaxValue
      LESeq = DateSeq_MinValue
      TotalArea = 0.0
      DO Sub = Sub1, Sub2
         WRITE(SubFile, 1001) LakeName3(LkNum), Sub
         OPEN(UNIT=11, FILE=TRIM(SubFile), STATUS='OLD', ERR=811)
         READ(11, *, ERR=812)      ! descriptive text
         READ(11, *, ERR=812)      ! lake basin name
         READ(11, *, ERR=812)      ! subbasin
         
         READ(11, 1100, ERR=812) Line
         CALL ParseCommaSepLine(Line, Strings, NumStrings);  IF (ErrorLevel .NE. 0) GOTO 851
         IF (NumStrings .LT. 2) GOTO 851
         READ(Strings(2), *, ERR=851) DV
         IF ((DV .LT. 0) .OR. (DV .GT. 9.9e13)) GOTO 851
         TotalArea = TotalArea + DV
         
         READ(11, 1100, ERR=812) Line
         CALL ParseCommaSepLine(Line, Strings, NumStrings);  IF (ErrorLevel .NE. 0) GOTO 851
         IF (NumStrings .LT. 2) GOTO 851
         SSeq = DateStringYMDToSeq(Strings(2));  IF (ErrorLevel .NE. 0) GOTO 851
         IF (SSeq .EQ. MissingData_Date) SSeq = DateSeq_MaxValue

         READ(11, 1100, ERR=812) Line
         CALL ParseCommaSepLine(Line, Strings, NumStrings);  IF (ErrorLevel .NE. 0) GOTO 851
         IF (NumStrings .LT. 2) GOTO 851
         ESeq = DateStringYMDToSeq(Strings(2));  IF (ErrorLevel .NE. 0) GOTO 851
         IF (ESeq .EQ. MissingData_Date) ESeq = DateSeq_MinValue
         CLOSE(11)

         IF (SSeq .LT. LSSeq) LSSeq = SSeq
         IF (ESeq .GT. LESeq) LESeq = ESeq
      END DO
      NumDays = LESeq - LSSeq + 1
      SDate = SeqToDateStringYMD(LSSeq)
      EDate = SeqToDateStringYMD(LESeq)
      PRINT*, 'Period is '//TRIM(SDate)//' to '//TRIM(EDate)//';  days=', NumDays
      
      !
      !  Allocate the arrays
      !
      ALLOCATE(SubData(7,NumDays,Sub1:Sub2), LumpData(7,NumDays), STAT=IOS)
      IF (IOS .NE. 0) THEN
         PRINT*, 'Unable to allocate RAM for ', NumDays, ' days of data'
         GOTO 898
      END IF      
      SubData(:,:,:) = MissingData_Real
      LumpData(:,:)  = MissingData_Real
      
      !
      !  Now read the data from the subdata_xxxnn.csv files
      !  I am taking a minimalist approach to the error-checking of the file here.
      !  In GLSHFS there is much more comprehensive parsing, etc. But I don't
      !  want make this program code that complex, etc.  So I am just going to 
      !  *assume* the file is set up in standard format, and do minimal checking
      !  to verify that is the case.  If not, error handling will be pretty
      !  abrupt and minimal.
      !
      DO Sub = Sub1, Sub2
         WRITE(SubFile, 1001) LakeName3(LkNum), Sub
         PRINT*, 'Reading data from ', TRIM(SubFile)
         OPEN(UNIT=11, FILE=TRIM(SubFile), STATUS='OLD', ERR=811)
         READ(11, *, ERR=812)
         READ(11, 1100, ERR=812) Line
         CALL ParseCommaSepLine(Line, Strings, NumStrings);  IF (ErrorLevel .NE. 0) GOTO 851
         IF (NumStrings .LT. 2) GOTO 851
         IF (TRIM(Strings(2)) .NE. LakeName3(LkNum)) GOTO 851
         
         READ(11, 1100, ERR=812) Line
         CALL ParseCommaSepLine(Line, Strings, NumStrings);  IF (ErrorLevel .NE. 0) GOTO 851
         IF (NumStrings .LT. 2) GOTO 851
         CALL LowerCase(Strings(2))
         IF (TRIM(ADJUSTL(Strings(2))) .EQ. 'lake') THEN
            S = 0
         ELSE
            READ(Strings(2), *, ERR=851) S
         END IF
         IF (S .NE. Sub) GOTO 851

         READ(11, 1100, ERR=812) Line
         CALL ParseCommaSepLine(Line, Strings, NumStrings);  IF (ErrorLevel .NE. 0) GOTO 851
         IF (NumStrings .LT. 2) GOTO 851
         READ(Strings(2), *, ERR=851) DV
         IF ((DV .LT. 0) .OR. (DV .GT. 9.9e13)) GOTO 851
         SubArea(Sub) = DV

         READ(11, 1100, ERR=812) Line
         CALL ParseCommaSepLine(Line, Strings, NumStrings);  IF (ErrorLevel .NE. 0) GOTO 851
         IF (NumStrings .LT. 2) GOTO 851
         SSeq = DateStringYMDToSeq(Strings(2));  IF (ErrorLevel .NE. 0) GOTO 851

         READ(11, 1100, ERR=812) Line
         CALL ParseCommaSepLine(Line, Strings, NumStrings);  IF (ErrorLevel .NE. 0) GOTO 851
         IF (NumStrings .LT. 2) GOTO 851
         ESeq = DateStringYMDToSeq(Strings(2));  IF (ErrorLevel .NE. 0) GOTO 851

         READ(11, *, ERR=812)
         READ(11, *, ERR=812)
         READ(11, *, ERR=812)

         IF ((SSeq .NE. MissingData_Date) .AND. (ESeq .NE. MissingData_Date)) THEN
            DO Seq = SSeq, ESeq
               I = Seq - LSSeq + 1
               READ(11, 1100, ERR=812) Line
               CALL ParseCommaSepLine(Line, Strings, NumStrings);  IF (ErrorLevel .NE. 0) GOTO 851
               IF (NumStrings .NE. 8) GOTO 851
               S = DateStringYMDToSeq(Strings(1))
               IF (S .EQ. Seq) THEN
                  DO J = 1, 7
                     READ(Strings(J+1), *, IOSTAT=IOS) DV
                     IF (IOS .NE. 0) DV = MissingData_Real
                     SubData(J,I,Sub) = DV
                  END DO
               END IF
            END DO
         END IF
         CLOSE(11)
      END DO

      !
      !  Compute the weighted average value for each day
      !
      PRINT*, 'Computing the weighted average values'
      DO Seq = LSSeq, LESeq
         I = Seq - LSSeq + 1
         DO J = 1, 7
            TotV = 0.0
            TotA = 0.0
            DO Sub = Sub1, Sub2
               IF (SubData(J,I,Sub) .GT. MissingData_Real_Test) THEN
                  TotV = TotV + (SubData(J,I,Sub) * SubArea(Sub))
                  TotA = TotA + SubArea(Sub)
               END IF
            END DO
            IF (TotA .GT. 1.0) THEN
               LumpData(J,I) = TotV / TotA
            END IF
         END DO
      END DO
      
      !
      !     Write the output file
      !
      PRINT*, 'Writing output file ', TRIM(OutFile)
      OPEN(UNIT=21, FILE=TRIM(OutFile), STATUS='REPLACE', ERR=821)
      WRITE(21, 2101, ERR=823) TRIM(AreaName)
      WRITE(21, 2102, ERR=823) LakeName3(LkNum)
      WRITE(21, 2103, ERR=823) TRIM(AreaOut)
      WRITE(21, 2104, ERR=823) TotalArea
      WRITE(21, 2105, ERR=823) SeqToDateStringYMD(LSSeq)
      WRITE(21, 2106, ERR=823) SeqToDateStringYMD(LESeq)
      WRITE(21, 2107, ERR=823) SeqToDateStringYMD(LSSeq)
      WRITE(21, 2108, ERR=823)
      WRITE(21, 2109, ERR=823)
      DO Seq = LSSeq, LESeq
         I = Seq - LSSeq + 1
         OutStr(:) = '      NA'
         DO J = 1, 7
            IF (LumpData(J,I) .GT. MissingData_Real_Test) THEN
               WRITE(OutStr(J), 2150) LumpData(J,I)
            END IF
         END DO
         WRITE(21, 2110, ERR=823) SeqToDateStringYMD(Seq), (OutStr(J),J=1,7)
      END DO
      CLOSE(21)
      GOTO 999

!
!  Handlers for I/O errors
!
  811 ErrorMessage = 'Error opening file '//TRIM(SubFile); CALL PassMsg; GOTO 898
  812 ErrorMessage = 'Error reading file '//TRIM(SubFile); CALL PassMsg; GOTO 898
  821 ErrorMessage = 'Error opening file '//TRIM(OutFile); CALL PassMsg; GOTO 898
  823 ErrorMessage = 'Error writing file '//TRIM(OutFile); CALL PassMsg; GOTO 898

  
  851 ErrorMessage = 'Error when parsing the header info from '//TRIM(SubFile); CALL PassMsg
      ErrorMessage = 'Relevant line is: '//TRIM(Line);  CALL PassMsg
      GOTO 898      
      

  898 ErrorLevel = 1
      ErrorMessage = '[traceback] ComputeLumpedVals...'
      CALL PassMsg
      GOTO 999

  999 DEALLOCATE(SubData, STAT=IOS)
      DEALLOCATE(LumpData, STAT=IOS)
      RETURN

 1001 FORMAT('subdata_', A3, I2.2, '.csv')
 1002 FORMAT('subdata_', A3, '_', A4, '.csv')
 1100 FORMAT(A)
 2101 FORMAT('Daily data for ', A)
 2102 FORMAT('Lake:, ', A3)
 2103 FORMAT('Subbasin:, ', A)
 2104 FORMAT('Area:, ', E13.5E2)
 2105 FORMAT('Starts(YMD):, ', A)
 2106 FORMAT('Ends(YMD):, ', A)
 2107 FORMAT('NewDataStart(YMD):, ', A)
 2108 FORMAT('Date,AirTempMin,AirTempMax,Precipitation,AirTemp,Dewpoint,Windspeed,CloudCover')
 2109 FORMAT('YYYY-MM-DD,Celsius,Celsius,Millimeters,Celsius,Celsius,MetersPerSecond,Percent')
 2110 FORMAT(A10, 7(',', A))
 2150 FORMAT(F8.2)
 
      
      END SUBROUTINE ComputeLumpedVals

END MODULE LumpSubMet_Subs

!============================================================================
!============================================================================
!
!  This program reads the data from the subbasin meteorology files produced
!  by/for GLSHFS. It then computes a single weighted value (per day) and
!  outputs a weighted file of overLAND or overBASIN values. 
!
!  Input files are subdata_xxxnn.csv  (e.g. subdata_sup01.csv)
!  Output file is overland_met_xxx.csv or overbasin_met_xxx.csv
!----------------------------------------------------------------------------
      PROGRAM LUMP_SUBMET
      USE Glshfs_Util
      USE LumpSubMet_Subs
      IMPLICIT NONE
      INTEGER :: NumArgs, Surf
      CHARACTER (LEN=3)  :: Bsn
      CHARACTER (LEN=4)  :: AreaCode
      CHARACTER (LEN=25) :: LdBs
      CHARACTER (LEN=50) :: Command_Line
      CHARACTER (LEN=25), DIMENSION(5) :: ArgStrings

      !
      !  Get lake basin name and specify overland or overbasin
      !  Remember that ArgStrings(1) is the program name.
      !
      Bsn = '   '
      LdBs = '  '

      Command_Line = '          '
      CALL GET_COMMAND(Command_Line)
      CALL ParseCmdLine(Command_Line, ArgStrings, NumArgs)

      IF (NumArgs .LT. 3) GOTO 875
      Bsn = TRIM(ArgStrings(2))
      IF (LEN_TRIM(Bsn) .NE. 3) GOTO 875
      
      LdBs = TRIM(ArgStrings(3))
      CALL LowerCase(LdBs)
      Surf = -1
      IF (TRIM(LdBs) .EQ. 'ld')    Surf = 0
      IF (TRIM(LdBs) .EQ. 'land')  Surf = 0
      IF (TRIM(LdBs) .EQ. 'bs')    Surf = 1
      IF (TRIM(LdBs) .EQ. 'basn')  Surf = 1
      IF (TRIM(LdBs) .EQ. 'basin') Surf = 1

      IF (TRIM(LdBs) .EQ. 'lk')   Surf = 99
      IF (TRIM(LdBs) .EQ. 'lake') Surf = 99
      IF (Surf .EQ. 99) THEN
         PRINT*, 'Lumping just for overlake area is a non sequitor'
         PRINT*, 'Just use subbasin 0, which is the lake surface.'
         CALL EXIT(1)
      END IF
      
      IF (Surf .EQ. -1) GOTO 875
      IF (Surf .EQ. 0) AreaCode = 'land'
      IF (Surf .EQ. 1) AreaCode = 'basn'

      !
      !  Call the routine to read/process the data.
      !  Note that the variable I (error status number) is ignored here.
      !
      CALL ComputeLumpedVals(Bsn, AreaCode)
      CALL EXIT(0)

      
  875 ErrorMessage = 'USAGE: lump_submet <basin> <subarea>';    CALL PassMsg
      ErrorMessage = '   basin = sup, mic, hur, etc';           CALL PassMsg
      ErrorMessage = '   subarea = ld, land, bs, basn, basin';  CALL PassMsg
      ErrorMessage = '   e.g. lump_submet sup bs';              CALL PassMsg
      CALL EXIT(1)
      
      END PROGRAM LUMP_SUBMET
